"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class EventManager {
    _eventListeners;
    _maxListeners;
    constructor() {
        this._eventListeners = new Map();
        this._maxListeners = 100; // Default maximum number of listeners
    }
    maxListeners() {
        return this._maxListeners;
    }
    // Add an event listener
    addListener(event, listener) {
        this.on(event, listener);
    }
    on(event, listener) {
        if (!this._eventListeners.has(event)) {
            this._eventListeners.set(event, []);
        }
        const listeners = this._eventListeners.get(event);
        if (listeners) {
            if (listeners.length >= this._maxListeners) {
                console.warn(`MaxListenersExceededWarning: Possible event memory leak detected. ${listeners.length + 1} ${event} listeners added. Use setMaxListeners() to increase limit.`);
            }
            listeners.push(listener);
        }
    }
    // Remove an event listener
    removeListener(event, listener) {
        this.off(event, listener);
    }
    off(event, listener) {
        const listeners = this._eventListeners.get(event) ?? [];
        const index = listeners.indexOf(listener);
        if (index > -1) {
            listeners.splice(index, 1);
        }
        if (listeners.length === 0) {
            this._eventListeners.delete(event);
        }
    }
    // Emit an event
    emit(event, ...arguments_) {
        const listeners = this._eventListeners.get(event);
        if (listeners && listeners.length > 0) {
            for (const listener of listeners) {
                listener(...arguments_);
            }
        }
        else if (event === 'error') {
            // If it's an 'error' event with no listeners, throw the error.
            if (arguments_[0] instanceof Error) {
                throw arguments_[0]; // Throws the error object if the first arg is an error
            }
            else {
                const error = new CustomError(arguments_[0]);
                error.context = arguments_[0];
                throw error;
            }
        }
    }
    // Get all listeners for a specific event
    listeners(event) {
        return this._eventListeners.get(event) ?? [];
    }
    // Remove all listeners for a specific event
    removeAllListeners(event) {
        if (event) {
            this._eventListeners.delete(event);
        }
        else {
            this._eventListeners.clear();
        }
    }
    // Set the maximum number of listeners for a single event
    setMaxListeners(n) {
        this._maxListeners = n;
    }
}
class CustomError extends Error {
    context;
    constructor(message, context) {
        super(message);
        this.context = context;
        // Maintains proper stack trace for where our error was thrown (only available on V8)
        if (Error.captureStackTrace) {
            Error.captureStackTrace(this, CustomError);
        }
        this.name = this.constructor.name;
    }
}
exports.default = EventManager;
module.exports = EventManager;
//# sourceMappingURL=event-manager.js.map