"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyvHooks = void 0;
const serialize_1 = require("@keyv/serialize");
const hooks_manager_1 = __importDefault(require("./hooks-manager"));
const event_manager_1 = __importDefault(require("./event-manager"));
const stats_manager_1 = __importDefault(require("./stats-manager"));
var KeyvHooks;
(function (KeyvHooks) {
    KeyvHooks["PRE_SET"] = "preSet";
    KeyvHooks["POST_SET"] = "postSet";
    KeyvHooks["PRE_GET"] = "preGet";
    KeyvHooks["POST_GET"] = "postGet";
    KeyvHooks["PRE_GET_MANY"] = "preGetMany";
    KeyvHooks["POST_GET_MANY"] = "postGetMany";
    KeyvHooks["PRE_DELETE"] = "preDelete";
    KeyvHooks["POST_DELETE"] = "postDelete";
})(KeyvHooks || (exports.KeyvHooks = KeyvHooks = {}));
const iterableAdapters = [
    'sqlite',
    'postgres',
    'mysql',
    'mongo',
    'redis',
    'tiered',
];
class Keyv extends event_manager_1.default {
    opts;
    iterator;
    hooks = new hooks_manager_1.default();
    stats = new stats_manager_1.default(false);
    constructor(store, options) {
        super();
        options ??= {};
        store ??= {};
        this.opts = {
            namespace: 'keyv',
            serialize: serialize_1.defaultSerialize,
            deserialize: serialize_1.defaultDeserialize,
            emitErrors: true,
            // @ts-expect-error - Map is not a KeyvStoreAdapter
            store: new Map(),
            ...options,
        };
        if (store && store.get) {
            this.opts.store = store;
        }
        else {
            // @ts-expect-error - Map is not a KeyvStoreAdapter
            this.opts = {
                ...this.opts,
                ...store,
            };
        }
        if (this.opts.compression) {
            const compression = this.opts.compression;
            this.opts.serialize = compression.serialize.bind(compression);
            this.opts.deserialize = compression.deserialize.bind(compression);
        }
        if (this.opts.store) {
            if (typeof this.opts.store.on === 'function' && this.opts.emitErrors) {
                this.opts.store.on('error', (error) => this.emit('error', error));
            }
            this.opts.store.namespace = this.opts.namespace;
            // Attach iterators
            // @ts-ignore
            if (typeof this.opts.store[Symbol.iterator] === 'function' && this.opts.store instanceof Map) {
                this.iterator = this.generateIterator(this.opts.store);
            }
            else if ('iterator' in this.opts.store && this.opts.store.opts && this._checkIterableAdapter()) {
                this.iterator = this.generateIterator(this.opts.store.iterator.bind(this.opts.store));
            }
        }
        if (this.opts.stats) {
            this.stats.enabled = this.opts.stats;
        }
    }
    generateIterator(iterator) {
        const function_ = async function* () {
            for await (const [key, raw] of (typeof iterator === 'function'
                ? iterator(this.opts.store.namespace)
                : iterator)) {
                const data = await this.opts.deserialize(raw);
                if (this.opts.store.namespace && !key.includes(this.opts.store.namespace)) {
                    continue;
                }
                if (typeof data.expires === 'number' && Date.now() > data.expires) {
                    this.delete(key);
                    continue;
                }
                yield [this._getKeyUnprefix(key), data.value];
            }
        };
        return function_.bind(this);
    }
    _checkIterableAdapter() {
        return iterableAdapters.includes(this.opts.store.opts.dialect)
            || iterableAdapters.findIndex(element => this.opts.store.opts.url.includes(element)) >= 0;
    }
    _getKeyPrefix(key) {
        return `${this.opts.namespace}:${key}`;
    }
    _getKeyPrefixArray(keys) {
        return keys.map(key => `${this.opts.namespace}:${key}`);
    }
    _getKeyUnprefix(key) {
        return key
            .split(':')
            .splice(1)
            .join(':');
    }
    async get(key, options) {
        const { store } = this.opts;
        const isArray = Array.isArray(key);
        const keyPrefixed = isArray ? this._getKeyPrefixArray(key) : this._getKeyPrefix(key);
        const isDataExpired = (data) => typeof data.expires === 'number' && Date.now() > data.expires;
        if (isArray) {
            this.hooks.trigger(KeyvHooks.PRE_GET_MANY, { keys: keyPrefixed });
            if (store.getMany === undefined) {
                const promises = keyPrefixed.map(async (key) => {
                    const rawData = await store.get(key);
                    const deserializedRow = (typeof rawData === 'string' || this.opts.compression) ? await this.opts.deserialize(rawData) : rawData;
                    if (deserializedRow === undefined || deserializedRow === null) {
                        return undefined;
                    }
                    if (isDataExpired(deserializedRow)) {
                        await this.delete(key);
                        return undefined;
                    }
                    return (options?.raw) ? deserializedRow : deserializedRow.value;
                });
                const deserializedRows = await Promise.allSettled(promises);
                const result = deserializedRows.map(row => row.value);
                this.hooks.trigger(KeyvHooks.POST_GET_MANY, result);
                if (result.length > 0) {
                    this.stats.hit();
                }
                return result;
            }
            const rawData = await store.getMany(keyPrefixed);
            const result = [];
            for (const index in rawData) {
                let row = rawData[index];
                if ((typeof row === 'string')) {
                    row = await this.opts.deserialize(row);
                }
                if (row === undefined || row === null) {
                    result.push(undefined);
                    continue;
                }
                if (isDataExpired(row)) {
                    await this.delete(key[index]);
                    result.push(undefined);
                    continue;
                }
                const value = (options?.raw) ? row : row.value;
                result.push(value);
            }
            this.hooks.trigger(KeyvHooks.POST_GET_MANY, result);
            if (result.length > 0) {
                this.stats.hit();
            }
            return result;
        }
        this.hooks.trigger(KeyvHooks.PRE_GET, { key: keyPrefixed });
        const rawData = await store.get(keyPrefixed);
        const deserializedData = (typeof rawData === 'string' || this.opts.compression) ? await this.opts.deserialize(rawData) : rawData;
        if (deserializedData === undefined || deserializedData === null) {
            this.stats.miss();
            return undefined;
        }
        if (isDataExpired(deserializedData)) {
            await this.delete(key);
            this.stats.miss();
            return undefined;
        }
        this.hooks.trigger(KeyvHooks.POST_GET, { key: keyPrefixed, value: deserializedData });
        this.stats.hit();
        return (options?.raw) ? deserializedData : deserializedData.value;
    }
    async set(key, value, ttl) {
        this.hooks.trigger(KeyvHooks.PRE_SET, { key, value, ttl });
        const keyPrefixed = this._getKeyPrefix(key);
        if (typeof ttl === 'undefined') {
            ttl = this.opts.ttl;
        }
        if (ttl === 0) {
            ttl = undefined;
        }
        const { store } = this.opts;
        const expires = (typeof ttl === 'number') ? (Date.now() + ttl) : null;
        if (typeof value === 'symbol') {
            this.emit('error', 'symbol cannot be serialized');
        }
        value = { value, expires };
        value = await this.opts.serialize(value);
        await store.set(keyPrefixed, value, ttl);
        this.hooks.trigger(KeyvHooks.POST_SET, { key: keyPrefixed, value, ttl });
        this.stats.set();
        return true;
    }
    async delete(key) {
        const { store } = this.opts;
        if (Array.isArray(key)) {
            const keyPrefixed = this._getKeyPrefixArray(key);
            this.hooks.trigger(KeyvHooks.PRE_DELETE, { key: keyPrefixed });
            if (store.deleteMany !== undefined) {
                return store.deleteMany(keyPrefixed);
            }
            const promises = keyPrefixed.map(async (key) => store.delete(key));
            const results = await Promise.allSettled(promises);
            const returnResult = results.every(x => x.value === true);
            this.hooks.trigger(KeyvHooks.POST_DELETE, returnResult);
            return returnResult;
        }
        const keyPrefixed = this._getKeyPrefix(key);
        const result = store.delete(keyPrefixed);
        this.hooks.trigger(KeyvHooks.POST_DELETE, result);
        this.stats.delete();
        return result;
    }
    async clear() {
        this.emit('clear');
        const { store } = this.opts;
        await store.clear();
    }
    async has(key) {
        const keyPrefixed = this._getKeyPrefix(key);
        const { store } = this.opts;
        if (store.has !== undefined && !(store instanceof Map)) {
            return store.has(keyPrefixed);
        }
        const rawData = await store.get(keyPrefixed);
        if (rawData) {
            const data = this.opts.deserialize(rawData);
            if (data) {
                if (data.expires === undefined || data.expires === null) {
                    return true;
                }
                return data.expires > Date.now();
            }
        }
        return false;
    }
    async disconnect() {
        const { store } = this.opts;
        this.emit('disconnect');
        if (typeof store.disconnect === 'function') {
            return store.disconnect();
        }
    }
}
exports.default = Keyv;
module.exports = Keyv;
module.exports.KeyvHooks = KeyvHooks;
//# sourceMappingURL=index.js.map